@@ -5,6 +5,7 @@ A graph is chordal if every cycle of length at least 4 has a chord
 (an edge joining two nodes not adjacent in the cycle).
 https://en.wikipedia.org/wiki/Chordal_graph
 """
+
 import sys
 
 import networkx as nx
@@ -28,6 +29,7 @@ class NetworkXTreewidthBoundExceeded(nx.NetworkXException):
 
 @not_implemented_for("directed")
 @not_implemented_for("multigraph")
+@nx._dispatchable
 def is_chordal(G):
     """Checks whether G is a chordal graph.
 
@@ -73,6 +75,8 @@ def is_chordal(G):
     search. It returns False when it finds that the separator for any node
     is not a clique.  Based on the algorithms in [1]_.
 
+    Self loops are ignored.
+
     References
     ----------
     .. [1] R. E. Tarjan and M. Yannakakis, Simple linear-time algorithms
@@ -80,9 +84,12 @@ def is_chordal(G):
        selectively reduce acyclic hypergraphs, SIAM J. Comput., 13 (1984),
        pp. 566–579.
     """
+    if len(G.nodes) <= 3:
+        return True
     return len(_find_chordality_breaker(G)) == 0
 
 
+@nx._dispatchable
 def find_induced_nodes(G, s, t, treewidth_bound=sys.maxsize):
     """Returns the set of induced nodes in the path from s to t.
 
@@ -130,6 +137,8 @@ def find_induced_nodes(G, s, t, treewidth_bound=sys.maxsize):
     The algorithm is inspired by Algorithm 4 in [1]_.
     A formal definition of induced node can also be found on that reference.
 
+    Self Loops are ignored
+
     References
     ----------
     .. [1] Learning Bounded Treewidth Bayesian Networks.
@@ -160,6 +169,7 @@ def find_induced_nodes(G, s, t, treewidth_bound=sys.maxsize):
     return induced_nodes
 
 
+@nx._dispatchable
 def chordal_graph_cliques(G):
     """Returns all maximal cliques of a chordal graph.
 
@@ -232,6 +242,7 @@ def chordal_graph_cliques(G):
             yield frozenset(clique_wanna_be)
 
 
+@nx._dispatchable
 def chordal_graph_treewidth(G):
     """Returns the treewidth of the chordal graph G.
 
@@ -326,9 +337,11 @@ def _find_chordality_breaker(G, s=None, treewidth_bound=sys.maxsize):
 
     If it does find one, it returns (u,v,w) where u,v,w are the three
     nodes that together with s are involved in the cycle.
+
+    It ignores any self loops.
     """
-    if nx.number_of_selfloops(G) > 0:
-        raise nx.NetworkXError("Input graph is not chordal.")
+    if len(G) == 0:
+        raise nx.NetworkXPointlessConcept("Graph has no nodes.")
     unnumbered = set(G)
     if s is None:
         s = arbitrary_element(G)
@@ -357,6 +370,7 @@ def _find_chordality_breaker(G, s=None, treewidth_bound=sys.maxsize):
 
 
 @not_implemented_for("directed")
+@nx._dispatchable(returns_graph=True)
 def complete_to_chordal_graph(G):
     """Return a copy of G completed to a chordal graph
 
