@@ -6,6 +6,8 @@ import warnings
 
 import pytest
 
+import networkx as nx
+
 mpl = pytest.importorskip("matplotlib")
 np = pytest.importorskip("numpy")
 mpl.use("PS")
@@ -13,45 +15,566 @@ plt = pytest.importorskip("matplotlib.pyplot")
 plt.rcParams["text.usetex"] = False
 
 
-import networkx as nx
-
 barbell = nx.barbell_graph(4, 6)
 
+defaults = {
+    "node_pos": None,
+    "node_visible": True,
+    "node_color": "#1f78b4",
+    "node_size": 300,
+    "node_label": {
+        "size": 12,
+        "color": "#000000",
+        "family": "sans-serif",
+        "weight": "normal",
+        "alpha": 1.0,
+        "background_color": None,
+        "background_alpha": None,
+        "h_align": "center",
+        "v_align": "center",
+        "bbox": None,
+    },
+    "node_shape": "o",
+    "node_alpha": 1.0,
+    "node_border_width": 1.0,
+    "node_border_color": "face",
+    "edge_visible": True,
+    "edge_width": 1.0,
+    "edge_color": "#000000",
+    "edge_label": {
+        "size": 12,
+        "color": "#000000",
+        "family": "sans-serif",
+        "weight": "normal",
+        "alpha": 1.0,
+        "bbox": {"boxstyle": "round", "ec": (1.0, 1.0, 1.0), "fc": (1.0, 1.0, 1.0)},
+        "h_align": "center",
+        "v_align": "center",
+        "pos": 0.5,
+        "rotate": True,
+    },
+    "edge_style": "-",
+    "edge_alpha": 1.0,
+    # These are for undirected-graphs. Directed graphs shouls use "-|>" and 10, respectively
+    "edge_arrowstyle": "-",
+    "edge_arrowsize": 0,
+    "edge_curvature": "arc3",
+    "edge_source_margin": 0,
+    "edge_target_margin": 0,
+}
 
-def test_draw():
-    try:
-        functions = [
-            nx.draw_circular,
-            nx.draw_kamada_kawai,
-            nx.draw_planar,
-            nx.draw_random,
-            nx.draw_spectral,
-            nx.draw_spring,
-            nx.draw_shell,
+
+@pytest.mark.parametrize(
+    ("param_name", "param_value", "expected"),
+    (
+        ("node_color", None, defaults["node_color"]),
+        ("node_color", "#FF0000", "red"),
+        ("node_color", "color", "lime"),
+    ),
+)
+def test_display_arg_handling_node_color(param_name, param_value, expected):
+    G = nx.path_graph(4)
+    nx.set_node_attributes(G, "#00FF00", "color")
+    canvas = plt.figure().add_subplot(111)
+    nx.display(G, canvas=canvas, **{param_name: param_value})
+    assert mpl.colors.same_color(canvas.get_children()[0].get_edgecolors()[0], expected)
+    plt.close()
+
+
+@pytest.mark.parametrize(
+    ("param_value", "expected"),
+    (
+        (None, (1, 1, 1, 1)),  # default value
+        (0.5, (0.5, 0.5, 0.5, 0.5)),
+        ("n_alpha", (1.0, 1 / 2, 1 / 3, 0.25)),
+    ),
+)
+def test_display_arg_handling_node_alpha(param_value, expected):
+    G = nx.path_graph(4)
+    nx.set_node_attributes(G, {n: 1 / (n + 1) for n in G.nodes()}, "n_alpha")
+    canvas = plt.figure().add_subplot(111)
+    nx.display(G, canvas=canvas, node_alpha=param_value)
+    assert all(
+        canvas.get_children()[0].get_fc()[:, 3] == expected
+    )  # Extract just the alpha from the node colors
+    plt.close()
+
+
+def test_display_node_position():
+    G = nx.path_graph(4)
+    nx.set_node_attributes(G, {n: (n, n) for n in G.nodes()}, "pos")
+    canvas = plt.figure().add_subplot(111)
+    nx.display(G, canvas=canvas, node_pos="pos")
+    assert np.all(
+        canvas.get_children()[0].get_offsets().data == [[0, 0], [1, 1], [2, 2], [3, 3]]
+    )
+    plt.close()
+
+
+def test_display_line_collection():
+    G = nx.karate_club_graph()
+    nx.set_edge_attributes(
+        G, {(u, v): "-|>" if (u + v) % 2 else "-" for u, v in G.edges()}, "arrowstyle"
+    )
+    canvas = plt.figure().add_subplot(111)
+    nx.display(G, canvas=canvas, edge_arrowsize=10)
+    # There should only be one line collection in any given visualization
+    lc = [
+        l
+        for l in canvas.get_children()
+        if isinstance(l, mpl.collections.LineCollection)
+    ][0]
+    assert len(lc.get_paths()) == sum([1 for u, v in G.edges() if (u + v) % 2])
+    plt.close()
+
+
+@pytest.mark.parametrize(
+    ("edge_color", "expected"),
+    (
+        (None, "black"),
+        ("r", "red"),
+        ((1.0, 1.0, 0.0), "yellow"),
+        ((0, 1, 0, 1), "lime"),
+        ("color", "blue"),
+        ("#0000FF", "blue"),
+    ),
+)
+@pytest.mark.parametrize("graph_type", (nx.Graph, nx.DiGraph))
+def test_display_edge_single_color(edge_color, expected, graph_type):
+    G = nx.path_graph(3, create_using=graph_type)
+    nx.set_edge_attributes(G, "#0000FF", "color")
+    canvas = plt.figure().add_subplot(111)
+    nx.display(G, edge_color=edge_color, canvas=canvas)
+    if G.is_directed():
+        colors = [
+            f.get_fc()
+            for f in canvas.get_children()
+            if isinstance(f, mpl.patches.FancyArrowPatch)
         ]
-        options = [{"node_color": "black", "node_size": 100, "width": 3}]
-        for function, option in itertools.product(functions, options):
-            function(barbell, **option)
-            plt.savefig("test.ps")
-    except ModuleNotFoundError:  # draw_kamada_kawai requires scipy
-        pass
-    finally:
-        try:
-            os.unlink("test.ps")
-        except OSError:
-            pass
+    else:
+        colors = [
+            l
+            for l in canvas.collections
+            if isinstance(l, mpl.collections.LineCollection)
+        ][0].get_colors()
+    assert all(mpl.colors.same_color(c, expected) for c in colors)
+    plt.close()
 
 
-def test_draw_shell_nlist():
-    try:
-        nlist = [list(range(4)), list(range(4, 10)), list(range(10, 14))]
-        nx.draw_shell(barbell, nlist=nlist)
-        plt.savefig("test.ps")
-    finally:
-        try:
-            os.unlink("test.ps")
-        except OSError:
-            pass
+@pytest.mark.parametrize("graph_type", (nx.Graph, nx.DiGraph))
+def test_display_edge_multiple_colors(graph_type):
+    G = nx.path_graph(3, create_using=graph_type)
+    nx.set_edge_attributes(G, {(0, 1): "#FF0000", (1, 2): (0, 0, 1)}, "color")
+    ax = plt.figure().add_subplot(111)
+    nx.display(G, canvas=ax)
+    expected = ["red", "blue"]
+    if G.is_directed():
+        colors = [
+            f.get_fc()
+            for f in ax.get_children()
+            if isinstance(f, mpl.patches.FancyArrowPatch)
+        ]
+    else:
+        colors = [
+            l for l in ax.collections if isinstance(l, mpl.collections.LineCollection)
+        ][0].get_colors()
+    assert mpl.colors.same_color(colors, expected)
+    plt.close()
+
+
+@pytest.mark.parametrize("graph_type", (nx.Graph, nx.DiGraph))
+def test_display_edge_position(graph_type):
+    G = nx.path_graph(3, create_using=graph_type)
+    nx.set_node_attributes(G, {n: (n, n) for n in G.nodes()}, "pos")
+    ax = plt.figure().add_subplot(111)
+    nx.display(G, canvas=ax)
+    if G.is_directed():
+        end_points = [
+            (f.get_path().vertices[0, :], f.get_path().vertices[-2, :])
+            for f in ax.get_children()
+            if isinstance(f, mpl.patches.FancyArrowPatch)
+        ]
+    else:
+        line_collection = [
+            l for l in ax.collections if isinstance(l, mpl.collections.LineCollection)
+        ][0]
+        end_points = [
+            (p.vertices[0, :], p.vertices[-1, :]) for p in line_collection.get_paths()
+        ]
+    expected = [((0, 0), (1, 1)), ((1, 1), (2, 2))]
+    # Use the threshold to account for slight shifts in FancyArrowPatch margins to
+    # avoid covering the arrow head with the node.
+    threshold = 0.05
+    for a, e in zip(end_points, expected):
+        act_start, act_end = a
+        exp_start, exp_end = e
+        assert all(abs(act_start - exp_start) < (threshold, threshold)) and all(
+            abs(act_end - exp_end) < (threshold, threshold)
+        )
+    plt.close()
+
+
+def test_display_position_function():
+    G = nx.karate_club_graph()
+
+    def fixed_layout(G):
+        return nx.spring_layout(G, seed=314159)
+
+    pos = fixed_layout(G)
+    ax = plt.figure().add_subplot(111)
+    nx.display(G, node_pos=fixed_layout, canvas=ax)
+    # rebuild the position dictionary from the canvas
+    act_pos = {
+        n: tuple(p) for n, p in zip(G.nodes(), ax.get_children()[0].get_offsets().data)
+    }
+    for n in G.nodes():
+        assert all(pos[n] == act_pos[n])
+    plt.close()
+
+
+@pytest.mark.parametrize("graph_type", (nx.Graph, nx.DiGraph))
+def test_display_edge_colormaps(graph_type):
+    G = nx.path_graph(3, create_using=graph_type)
+    nx.set_edge_attributes(G, {(0, 1): 0, (1, 2): 1}, "weight")
+    cmap = mpl.colormaps["plasma"]
+    nx.apply_matplotlib_colors(G, "weight", "color", cmap, nodes=False)
+    canvas = plt.figure().add_subplot(111)
+    nx.display(G, canvas=canvas)
+    mapper = mpl.cm.ScalarMappable(cmap=cmap)
+    mapper.set_clim(0, 1)
+    expected = [mapper.to_rgba(0), mapper.to_rgba(1)]
+    if G.is_directed():
+        colors = [
+            f.get_facecolor()
+            for f in canvas.get_children()
+            if isinstance(f, mpl.patches.FancyArrowPatch)
+        ]
+    else:
+        colors = [
+            l
+            for l in canvas.collections
+            if isinstance(l, mpl.collections.LineCollection)
+        ][0].get_colors()
+    assert mpl.colors.same_color(expected[0], G.edges[0, 1]["color"])
+    assert mpl.colors.same_color(expected[1], G.edges[1, 2]["color"])
+    assert mpl.colors.same_color(expected, colors)
+    plt.close()
+
+
+@pytest.mark.parametrize("graph_type", (nx.Graph, nx.DiGraph))
+def test_display_node_colormaps(graph_type):
+    G = nx.path_graph(3, create_using=graph_type)
+    nx.set_node_attributes(G, {0: 0, 1: 0.5, 2: 1}, "weight")
+    cmap = mpl.colormaps["plasma"]
+    nx.apply_matplotlib_colors(G, "weight", "color", cmap)
+    canvas = plt.figure().add_subplot(111)
+    nx.display(G, canvas=canvas)
+    mapper = mpl.cm.ScalarMappable(cmap=cmap)
+    mapper.set_clim(0, 1)
+    expected = [mapper.to_rgba(0), mapper.to_rgba(0.5), mapper.to_rgba(1)]
+    colors = [
+        s for s in canvas.collections if isinstance(s, mpl.collections.PathCollection)
+    ][0].get_edgecolors()
+    assert mpl.colors.same_color(expected[0], G.nodes[0]["color"])
+    assert mpl.colors.same_color(expected[1], G.nodes[1]["color"])
+    assert mpl.colors.same_color(expected[2], G.nodes[2]["color"])
+    assert mpl.colors.same_color(expected, colors)
+    plt.close()
+
+
+@pytest.mark.parametrize(
+    ("param_value", "expected"),
+    (
+        (None, [defaults["edge_width"], defaults["edge_width"]]),
+        (5, [5, 5]),
+        ("width", [5, 10]),
+    ),
+)
+@pytest.mark.parametrize("graph_type", (nx.Graph, nx.DiGraph))
+def test_display_edge_width(param_value, expected, graph_type):
+    G = nx.path_graph(3, create_using=graph_type)
+    nx.set_edge_attributes(G, {(0, 1): 5, (1, 2): 10}, "width")
+    canvas = plt.figure().add_subplot(111)
+    nx.display(G, edge_width=param_value, canvas=canvas)
+    if G.is_directed():
+        widths = [
+            f.get_linewidth()
+            for f in canvas.get_children()
+            if isinstance(f, mpl.patches.FancyArrowPatch)
+        ]
+    else:
+        widths = list(
+            [
+                l
+                for l in canvas.collections
+                if isinstance(l, mpl.collections.LineCollection)
+            ][0].get_linewidths()
+        )
+    assert widths == expected
+
+
+@pytest.mark.parametrize(
+    ("param_value", "expected"),
+    (
+        (None, [defaults["edge_style"], defaults["edge_style"]]),
+        (":", [":", ":"]),
+        ("style", ["-", ":"]),
+    ),
+)
+@pytest.mark.parametrize("graph_type", (nx.Graph, nx.DiGraph))
+def test_display_edge_style(param_value, expected, graph_type):
+    G = nx.path_graph(3, create_using=graph_type)
+    nx.set_edge_attributes(G, {(0, 1): "-", (1, 2): ":"}, "style")
+    canvas = plt.figure().add_subplot(111)
+    nx.display(G, edge_style=param_value, canvas=canvas)
+    if G.is_directed():
+        styles = [
+            f.get_linestyle()
+            for f in canvas.get_children()
+            if isinstance(f, mpl.patches.FancyArrowPatch)
+        ]
+    else:
+        # Convert back from tuple description to character form
+        linestyles = {(0, None): "-", (0, (1, 1.65)): ":"}
+        styles = [
+            linestyles[(s[0], tuple(s[1]) if s[1] is not None else None)]
+            for s in [
+                l
+                for l in canvas.collections
+                if isinstance(l, mpl.collections.LineCollection)
+            ][0].get_linestyles()
+        ]
+    assert styles == expected
+    plt.close()
+
+
+def test_display_node_labels():
+    G = nx.path_graph(4)
+    canvas = plt.figure().add_subplot(111)
+    nx.display(G, canvas=canvas, node_label={"size": 20})
+    labels = [t for t in canvas.get_children() if isinstance(t, mpl.text.Text)]
+    for n, l in zip(G.nodes(), labels):
+        assert l.get_text() == str(n)
+        assert l.get_size() == 20.0
+    plt.close()
+
+
+def test_display_edge_labels():
+    G = nx.path_graph(4)
+    canvas = plt.figure().add_subplot(111)
+    # While we can pass in dicts for edge label defaults without errors,
+    # this isn't helpful unless we want one label for all edges.
+    nx.set_edge_attributes(G, {(u, v): {"label": u + v} for u, v in G.edges()})
+    nx.display(G, canvas=canvas, edge_label={"color": "r"}, node_label=None)
+    labels = [t for t in canvas.get_children() if isinstance(t, mpl.text.Text)]
+    print(labels)
+    for e, l in zip(G.edges(), labels):
+        assert l.get_text() == str(e[0] + e[1])
+        assert l.get_color() == "r"
+    plt.close()
+
+
+def test_display_multigraph_non_integer_keys():
+    G = nx.MultiGraph()
+    G.add_nodes_from(["A", "B", "C", "D"])
+    G.add_edges_from(
+        [
+            ("A", "B", "0"),
+            ("A", "B", "1"),
+            ("B", "C", "-1"),
+            ("B", "C", "1"),
+            ("C", "D", "-1"),
+            ("C", "D", "0"),
+        ]
+    )
+    nx.set_edge_attributes(
+        G, {e: f"arc3,rad={0.2 * int(e[2])}" for e in G.edges(keys=True)}, "curvature"
+    )
+    canvas = plt.figure().add_subplot(111)
+    nx.display(G, canvas=canvas)
+    rads = [
+        f.get_connectionstyle().rad
+        for f in canvas.get_children()
+        if isinstance(f, mpl.patches.FancyArrowPatch)
+    ]
+    assert rads == [0.0, 0.2, -0.2, 0.2, -0.2, 0.0]
+    plt.close()
+
+
+def test_display_raises_for_bad_arg():
+    G = nx.karate_club_graph()
+    with pytest.raises(nx.NetworkXError):
+        nx.display(G, bad_arg="bad_arg")
+        plt.close()
+
+
+def test_display_arrow_size():
+    G = nx.path_graph(4, create_using=nx.DiGraph)
+    nx.set_edge_attributes(
+        G, {(u, v): (u + v + 2) ** 2 for u, v in G.edges()}, "arrowsize"
+    )
+    ax = plt.axes()
+    nx.display(G, canvas=ax)
+    assert [9, 25, 49] == [
+        f.get_mutation_scale()
+        for f in ax.get_children()
+        if isinstance(f, mpl.patches.FancyArrowPatch)
+    ]
+    plt.close()
+
+
+def test_display_mismatched_edge_position():
+    """
+    This test ensures that a error is raised for incomplete position data.
+    """
+    G = nx.path_graph(5)
+    # Notice that there is no position for node 3
+    nx.set_node_attributes(G, {0: (0, 0), 1: (1, 1), 2: (2, 2), 4: (4, 4)}, "pos")
+    # But that's not a problem since we don't want to show node 4, right?
+    nx.set_node_attributes(G, {n: n < 4 for n in G.nodes()}, "visible")
+    # However, if we try to visualize every edge (including 3 -> 4)...
+    # That's a problem since node 4 doesn't have a position
+    with pytest.raises(nx.NetworkXError):
+        nx.display(G)
+
+
+# NOTE: parametrizing on marker to test both branches of internal
+# to_marker_edge function
+@pytest.mark.parametrize("node_shape", ("o", "s"))
+def test_display_edge_margins(node_shape):
+    """
+    Test that there is a wider gap between the node and the start of an
+    incident edge when min_source_margin is specified.
+
+    This test checks that the use os min_{source/target}_margin edge
+    attributes result is shorter (more padding) between the edges and
+    source and target nodes.
+
+
+    As a crude visual example, let 's' and 't' represent source and target
+    nodes, respectively:
+
+       Default:
+       s-----------------------------t
+
+       With margins:
+       s   -----------------------   t
+
+    """
+    ax = plt.figure().add_subplot(111)
+    G = nx.DiGraph([(0, 1)])
+    nx.set_node_attributes(G, {0: (0, 0), 1: (1, 1)}, "pos")
+    # Get the default patches from the regular visualization
+    nx.display(G, canvas=ax, node_shape=node_shape)
+    default_arrow = [
+        f for f in ax.get_children() if isinstance(f, mpl.patches.FancyArrowPatch)
+    ][0]
+    default_extent = default_arrow.get_extents().corners()[::2, 0]
+    # Now plot again with margins
+    ax = plt.figure().add_subplot(111)
+    nx.display(
+        G,
+        canvas=ax,
+        edge_source_margin=100,
+        edge_target_margin=100,
+        node_shape=node_shape,
+    )
+    padded_arrow = [
+        f for f in ax.get_children() if isinstance(f, mpl.patches.FancyArrowPatch)
+    ][0]
+    padded_extent = padded_arrow.get_extents().corners()[::2, 0]
+
+    # With padding, the left-most extent of the edge should be further to the right
+    assert padded_extent[0] > default_extent[0]
+    # And the rightmost extent of the edge, further to the left
+    assert padded_extent[1] < default_extent[1]
+    plt.close()
+
+
+@pytest.mark.parametrize("ticks", [False, True])
+def test_display_hide_ticks(ticks):
+    G = nx.path_graph(3)
+    nx.set_node_attributes(G, {n: (n, n) for n in G.nodes()}, "pos")
+    ax = plt.axes()
+    nx.display(G, hide_ticks=ticks)
+    for axis in [ax.xaxis, ax.yaxis]:
+        assert bool(axis.get_ticklabels()) != ticks
+
+    plt.close()
+
+
+def test_display_self_loop():
+    ax = plt.axes()
+    G = nx.DiGraph()
+    G.add_node(0)
+    G.add_edge(0, 0)
+    nx.set_node_attributes(G, {0: (0, 0)}, "pos")
+    nx.display(G, canvas=ax)
+    arrow = [
+        f for f in ax.get_children() if isinstance(f, mpl.patches.FancyArrowPatch)
+    ][0]
+    bbox = arrow.get_extents()
+    print(bbox.width)
+    print(bbox.height)
+    assert bbox.width > 0 and bbox.height > 0
+
+    plt.delaxes(ax)
+    plt.close()
+
+
+def test_display_remove_pos_attr():
+    """
+    If the pos attribute isn't provided or is a function, display computes the layout
+    and adds it to the graph. We need to ensure that this new attribute is removed from
+    the returned graph.
+    """
+    G = nx.karate_club_graph()
+    nx.display(G)
+    assert nx.get_node_attributes(G, "display's position attribute name") == {}
+
+
+@pytest.fixture
+def subplots():
+    fig, ax = plt.subplots()
+    yield fig, ax
+    plt.delaxes(ax)
+    plt.close()
+
+
+@pytest.mark.parametrize(
+    "function",
+    [
+        nx.draw_circular,
+        nx.draw_kamada_kawai,
+        nx.draw_planar,
+        nx.draw_random,
+        nx.draw_spectral,
+        nx.draw_spring,
+        nx.draw_shell,
+        nx.draw_forceatlas2,
+    ],
+)
+def test_draw(function, subplots, tmp_path):
+    if function == nx.draw_kamada_kawai:
+        pytest.importorskip("scipy", reason="draw_kamada_kawai requires scipy")
+    fig, _ = subplots
+    options = {"node_color": "black", "node_size": 100, "width": 3}
+    function(barbell, **options)
+    fig.savefig(tmp_path / "test.ps")
+
+
+def test_draw_shell_nlist(subplots, tmp_path):
+    fig, _ = subplots
+    nlist = [list(range(4)), list(range(4, 10)), list(range(10, 14))]
+    nx.draw_shell(barbell, nlist=nlist)
+    fig.savefig(tmp_path / "test.ps")
+
+
+def test_draw_bipartite(subplots, tmp_path):
+    fig, _ = subplots
+    G = nx.complete_bipartite_graph(2, 5)
+    nx.draw_bipartite(G)
+    fig.savefig(tmp_path / "test.ps")
 
 
 def test_edge_colormap():
@@ -62,6 +585,16 @@ def test_edge_colormap():
     # plt.show()
 
 
+def test_draw_networkx_edge_labels(subplots, tmp_path):
+    fig, _ = subplots
+    edge = (0, 1)
+    G = nx.DiGraph([edge])
+    pos = {n: (n, n) for n in G}
+    nx.draw(G, pos=pos)
+    nx.draw_networkx_edge_labels(G, pos, edge_labels={edge: "edge"})
+    fig.savefig(tmp_path / "test.ps")
+
+
 def test_arrows():
     nx.draw_spring(barbell.to_directed())
     # plt.show()
@@ -362,9 +895,35 @@ def test_directed_edges_linestyle_sequence(style_seq):
         assert fap.get_linestyle() == style
 
 
+def test_return_types():
+    from matplotlib.collections import LineCollection, PathCollection
+    from matplotlib.patches import FancyArrowPatch
+
+    G = nx.frucht_graph(create_using=nx.Graph)
+    dG = nx.frucht_graph(create_using=nx.DiGraph)
+    pos = nx.spring_layout(G, seed=42)
+    dpos = nx.spring_layout(dG, seed=42)
+    # nodes
+    nodes = nx.draw_networkx_nodes(G, pos)
+    assert isinstance(nodes, PathCollection)
+    # edges
+    edges = nx.draw_networkx_edges(dG, dpos, arrows=True)
+    assert isinstance(edges, list)
+    if len(edges) > 0:
+        assert isinstance(edges[0], FancyArrowPatch)
+    edges = nx.draw_networkx_edges(dG, dpos, arrows=False)
+    assert isinstance(edges, LineCollection)
+    edges = nx.draw_networkx_edges(G, dpos, arrows=None)
+    assert isinstance(edges, LineCollection)
+    edges = nx.draw_networkx_edges(dG, pos, arrows=None)
+    assert isinstance(edges, list)
+    if len(edges) > 0:
+        assert isinstance(edges[0], FancyArrowPatch)
+
+
 def test_labels_and_colors():
     G = nx.cubical_graph()
-    pos = nx.spring_layout(G)  # positions for all nodes
+    pos = nx.spring_layout(G, seed=42)  # positions for all nodes
     # nodes
     nx.draw_networkx_nodes(
         G, pos, nodelist=[0, 1, 2, 3], node_color="r", node_size=500, alpha=0.75
@@ -423,34 +982,8 @@ def test_labels_and_colors():
     # plt.show()
 
 
-@pytest.mark.mpl_image_compare
-def test_house_with_colors():
-    G = nx.house_graph()
-    # explicitly set positions
-    fig, ax = plt.subplots()
-    pos = {0: (0, 0), 1: (1, 0), 2: (0, 1), 3: (1, 1), 4: (0.5, 2.0)}
-
-    # Plot nodes with different properties for the "wall" and "roof" nodes
-    nx.draw_networkx_nodes(
-        G,
-        pos,
-        node_size=3000,
-        nodelist=[0, 1, 2, 3],
-        node_color="tab:blue",
-    )
-    nx.draw_networkx_nodes(
-        G, pos, node_size=2000, nodelist=[4], node_color="tab:orange"
-    )
-    nx.draw_networkx_edges(G, pos, alpha=0.5, width=6)
-    # Customize axes
-    ax.margins(0.11)
-    plt.tight_layout()
-    plt.axis("off")
-    return fig
-
-
-def test_axes():
-    fig, ax = plt.subplots()
+def test_axes(subplots):
+    fig, ax = subplots
     nx.draw(barbell, ax=ax)
     nx.draw_networkx_edge_labels(barbell, nx.circular_layout(barbell), ax=ax)
 
@@ -507,9 +1040,9 @@ def test_alpha_iter():
     nx.draw_networkx_nodes(barbell, pos, alpha=alpha)
 
 
-def test_multiple_node_shapes():
+def test_multiple_node_shapes(subplots):
+    fig, ax = subplots
     G = nx.path_graph(4)
-    ax = plt.figure().add_subplot(111)
     nx.draw(G, node_shape=["o", "h", "s", "^"], ax=ax)
     scatters = [
         s for s in ax.get_children() if isinstance(s, mpl.collections.PathCollection)
@@ -517,9 +1050,9 @@ def test_multiple_node_shapes():
     assert len(scatters) == 4
 
 
-def test_individualized_font_attributes():
+def test_individualized_font_attributes(subplots):
     G = nx.karate_club_graph()
-    ax = plt.figure().add_subplot(111)
+    fig, ax = subplots
     nx.draw(
         G,
         ax=ax,
@@ -540,9 +1073,9 @@ def test_individualized_font_attributes():
         assert int(n / (34 / 15) + 5) == t.get_size()
 
 
-def test_individualized_edge_attributes():
+def test_individualized_edge_attributes(subplots):
     G = nx.karate_club_graph()
-    ax = plt.figure().add_subplot(111)
+    fig, ax = subplots
     arrowstyles = ["-|>" if (u + v) % 2 == 0 else "-[" for u, v in G.edges()]
     arrowsizes = [10 * (u % 2 + v % 2) + 10 for u, v in G.edges()]
     nx.draw(G, ax=ax, arrows=True, arrowstyle=arrowstyles, arrowsize=arrowsizes)
@@ -619,10 +1152,37 @@ def test_draw_nodes_missing_node_from_position():
         nx.draw_networkx_nodes(G, pos)
 
 
+def test_draw_networkx_nodes_node_shape_list_with_scalar_color(subplots):
+    """Ensure draw_networkx_nodes works when node_shape is a Python list.
+
+    This covers the case where node_shape is a sequence (list) and node_color
+    is a single scalar color, which should be supported.
+    """
+    fig, ax = subplots
+
+    G = nx.empty_graph(5)
+    pos = {i: (i, i) for i in G}
+
+    shapes = ["o", "^", "o", "^", "o"]
+
+    nodes = nx.draw_networkx_nodes(
+        G,
+        pos,
+        node_color="red",  # scalar color (supported)
+        node_shape=shapes,  # list of shapes – this used to be buggy
+        ax=ax,
+    )
+    # Should get a PathCollection with an element in it (same as with numpy arrays)
+    assert len(nodes.get_offsets()) > 0
+    # NOTE: When node_shape is a sequence, draw_networkx_nodes internally calls
+    # ax.scatter multiple times and returns only the last PathCollection.
+    # Therefore, we do NOT assert a value for len(nodes.get_offsets()) here.
+
+
 # NOTE: parametrizing on marker to test both branches of internal
 # nx.draw_networkx_edges.to_marker_edge function
 @pytest.mark.parametrize("node_shape", ("o", "s"))
-def test_draw_edges_min_source_target_margins(node_shape):
+def test_draw_edges_min_source_target_margins(node_shape, subplots):
     """Test that there is a wider gap between the node and the start of an
     incident edge when min_source_margin is specified.
 
@@ -640,7 +1200,7 @@ def test_draw_edges_min_source_target_margins(node_shape):
     """
     # Create a single axis object to get consistent pixel coords across
     # multiple draws
-    fig, ax = plt.subplots()
+    fig, ax = subplots
     G = nx.DiGraph([(0, 1)])
     pos = {0: (0, 0), 1: (1, 0)}  # horizontal layout
     # Get leftmost and rightmost points of the FancyArrowPatch object
@@ -669,7 +1229,7 @@ def test_draw_edges_min_source_target_margins(node_shape):
 # NOTE: parametrizing on marker to test both branches of internal
 # nx.draw_networkx_edges.to_marker_edge function
 @pytest.mark.parametrize("node_shape", ("o", "s"))
-def test_draw_edges_min_source_target_margins_individual(node_shape):
+def test_draw_edges_min_source_target_margins_individual(node_shape, subplots):
     """Test that there is a wider gap between the node and the start of an
     incident edge when min_source_margin is specified.
 
@@ -687,7 +1247,7 @@ def test_draw_edges_min_source_target_margins_individual(node_shape):
     """
     # Create a single axis object to get consistent pixel coords across
     # multiple draws
-    fig, ax = plt.subplots()
+    fig, ax = subplots
     G = nx.DiGraph([(0, 1), (1, 2)])
     pos = {0: (0, 0), 1: (1, 0), 2: (2, 0)}  # horizontal layout
     # Get leftmost and rightmost points of the FancyArrowPatch object
@@ -706,7 +1266,6 @@ def test_draw_edges_min_source_target_margins_individual(node_shape):
     )
     padded_extent = [p.get_extents().corners()[::2, 0] for p in padded_patch]
     for d, p in zip(default_extent, padded_extent):
-        print(f"{p=}, {d=}")
         # With padding, the left-most extent of the edge should be further to the
         # right
         assert p[0] > d[0]
@@ -714,10 +1273,10 @@ def test_draw_edges_min_source_target_margins_individual(node_shape):
         assert p[1] < d[1]
 
 
-def test_nonzero_selfloop_with_single_node():
+def test_nonzero_selfloop_with_single_node(subplots):
     """Ensure that selfloop extent is non-zero when there is only one node."""
     # Create explicit axis object for test
-    fig, ax = plt.subplots()
+    fig, ax = subplots
     # Graph with single node + self loop
     G = nx.DiGraph()
     G.add_node(0)
@@ -727,17 +1286,14 @@ def test_nonzero_selfloop_with_single_node():
     # The resulting patch must have non-zero extent
     bbox = patch.get_extents()
     assert bbox.width > 0 and bbox.height > 0
-    # Cleanup
-    plt.delaxes(ax)
-    plt.close()
 
 
-def test_nonzero_selfloop_with_single_edge_in_edgelist():
+def test_nonzero_selfloop_with_single_edge_in_edgelist(subplots):
     """Ensure that selfloop extent is non-zero when only a single edge is
     specified in the edgelist.
     """
     # Create explicit axis object for test
-    fig, ax = plt.subplots()
+    fig, ax = subplots
     # Graph with selfloop
     G = nx.path_graph(2, create_using=nx.DiGraph)
     G.add_edge(1, 1)
@@ -747,9 +1303,6 @@ def test_nonzero_selfloop_with_single_edge_in_edgelist():
     # The resulting patch must have non-zero extent
     bbox = patch.get_extents()
     assert bbox.width > 0 and bbox.height > 0
-    # Cleanup
-    plt.delaxes(ax)
-    plt.close()
 
 
 def test_apply_alpha():
@@ -798,42 +1351,36 @@ def test_draw_edges_toggling_with_arrows_kwarg():
 
 
 @pytest.mark.parametrize("drawing_func", (nx.draw, nx.draw_networkx))
-def test_draw_networkx_arrows_default_undirected(drawing_func):
+def test_draw_networkx_arrows_default_undirected(drawing_func, subplots):
     import matplotlib.collections
 
     G = nx.path_graph(3)
-    fig, ax = plt.subplots()
+    fig, ax = subplots
     drawing_func(G, ax=ax)
     assert any(isinstance(c, mpl.collections.LineCollection) for c in ax.collections)
     assert not ax.patches
-    plt.delaxes(ax)
-    plt.close()
 
 
 @pytest.mark.parametrize("drawing_func", (nx.draw, nx.draw_networkx))
-def test_draw_networkx_arrows_default_directed(drawing_func):
+def test_draw_networkx_arrows_default_directed(drawing_func, subplots):
     import matplotlib.collections
 
     G = nx.path_graph(3, create_using=nx.DiGraph)
-    fig, ax = plt.subplots()
+    fig, ax = subplots
     drawing_func(G, ax=ax)
     assert not any(
         isinstance(c, mpl.collections.LineCollection) for c in ax.collections
     )
     assert ax.patches
-    plt.delaxes(ax)
-    plt.close()
 
 
-def test_edgelist_kwarg_not_ignored():
+def test_edgelist_kwarg_not_ignored(subplots):
     # See gh-4994
     G = nx.path_graph(3)
     G.add_edge(0, 0)
-    fig, ax = plt.subplots()
+    fig, ax = subplots
     nx.draw(G, edgelist=[(0, 1), (1, 2)], ax=ax)  # Exclude self-loop from edgelist
     assert not ax.patches
-    plt.delaxes(ax)
-    plt.close()
 
 
 @pytest.mark.parametrize(
@@ -903,10 +1450,10 @@ def test_draw_networkx_edge_label_empty_dict():
     assert nx.draw_networkx_edge_labels(G, pos, edge_labels={}) == {}
 
 
-def test_draw_networkx_edges_undirected_selfloop_colors():
+def test_draw_networkx_edges_undirected_selfloop_colors(subplots):
     """When an edgelist is supplied along with a sequence of colors, check that
     the self-loops have the correct colors."""
-    fig, ax = plt.subplots()
+    fig, ax = subplots
     # Edge list and corresponding colors
     edgelist = [(1, 3), (1, 2), (2, 3), (1, 1), (3, 3), (2, 2)]
     edge_colors = ["pink", "cyan", "black", "red", "blue", "green"]
@@ -928,8 +1475,6 @@ def test_draw_networkx_edges_undirected_selfloop_colors():
     for fap, clr, slp in zip(ax.patches, edge_colors[-3:], sl_points):
         assert fap.get_path().contains_point(slp)
         assert mpl.colors.same_color(fap.get_edgecolor(), clr)
-    plt.delaxes(ax)
-    plt.close()
 
 
 @pytest.mark.parametrize(
@@ -942,13 +1487,13 @@ def test_draw_networkx_edges_undirected_selfloop_colors():
         {"min_target_margin": 10},
     ),
 )
-def test_user_warnings_for_unused_edge_drawing_kwargs(fap_only_kwarg):
+def test_user_warnings_for_unused_edge_drawing_kwargs(fap_only_kwarg, subplots):
     """Users should get a warning when they specify a non-default value for
     one of the kwargs that applies only to edges drawn with FancyArrowPatches,
     but FancyArrowPatches aren't being used under the hood."""
     G = nx.path_graph(3)
     pos = {n: (n, n) for n in G}
-    fig, ax = plt.subplots()
+    fig, ax = subplots
     # By default, an undirected graph will use LineCollection to represent
     # the edges
     kwarg_name = list(fap_only_kwarg.keys())[0]
@@ -961,24 +1506,19 @@ def test_user_warnings_for_unused_edge_drawing_kwargs(fap_only_kwarg):
     with warnings.catch_warnings():
         # Escalate warnings -> errors so tests fail if warnings are raised
         warnings.simplefilter("error")
+        warnings.filterwarnings("ignore", category=DeprecationWarning)
         nx.draw_networkx_edges(G, pos, ax=ax, arrows=True, **fap_only_kwarg)
 
-    plt.delaxes(ax)
-    plt.close()
-
 
 @pytest.mark.parametrize("draw_fn", (nx.draw, nx.draw_circular))
-def test_no_warning_on_default_draw_arrowstyle(draw_fn):
+def test_no_warning_on_default_draw_arrowstyle(draw_fn, subplots):
     # See gh-7284
-    fig, ax = plt.subplots()
+    fig, ax = subplots
     G = nx.cycle_graph(5)
     with warnings.catch_warnings(record=True) as w:
         draw_fn(G, ax=ax)
     assert len(w) == 0
 
-    plt.delaxes(ax)
-    plt.close()
-
 
 @pytest.mark.parametrize("hide_ticks", [False, True])
 @pytest.mark.parametrize(
@@ -991,13 +1531,52 @@ def test_no_warning_on_default_draw_arrowstyle(draw_fn):
         nx.draw_networkx_nodes,
     ],
 )
-def test_hide_ticks(method, hide_ticks):
+def test_hide_ticks(method, hide_ticks, subplots):
     G = nx.path_graph(3)
     pos = {n: (n, n) for n in G.nodes}
-    _, ax = plt.subplots()
+    _, ax = subplots
     method(G, pos=pos, ax=ax, hide_ticks=hide_ticks)
     for axis in [ax.xaxis, ax.yaxis]:
         assert bool(axis.get_ticklabels()) != hide_ticks
 
-    plt.delaxes(ax)
-    plt.close()
+
+@pytest.mark.parametrize(
+    "style", ["angle", "angle3", "arc", "arc3,rad=0.0", "bar,fraction=0.1"]
+)
+def test_edge_label_all_connectionstyles(subplots, style):
+    """
+    Check that FancyArrowPatches with all `connectionstyle`s are supported
+    in edge label rendering. See gh-7735 and gh-8106.
+    """
+    fig, ax = subplots
+    edge = (0, 1)
+    G = nx.DiGraph([edge])
+    pos = {n: (n, 0) for n in G}
+
+    name = style.split(",")[0]
+    labels = nx.draw_networkx_edge_labels(
+        G, pos, edge_labels={edge: "edge"}, connectionstyle=style
+    )
+
+    hmid = (pos[0][0] + pos[1][0]) / 2
+    vmid = (pos[0][1] + pos[1][1]) / 2
+    if name in {"arc", "arc3"}:  # The label should be at roughly the midpoint.
+        assert labels[edge].x, labels[edge].y == pytest.approx((hmid, vmid))
+    elif name == "bar":  # The label should be below the vertical midpoint.
+        assert labels[edge].y < vmid
+
+
+@pytest.mark.parametrize("label_pos", [-0.1, 1.1])
+def test_edge_label_label_pos(subplots, label_pos):
+    """
+    Check that label positions can be extrapolated outside [0, 1].
+    """
+    fig, ax = subplots
+    edge = (0, 1)
+    G = nx.DiGraph([edge])
+    pos = {n: (n, n) for n in G}
+    lbl = nx.draw_networkx_edge_labels(
+        G, pos, edge_labels={edge: "edge"}, label_pos=label_pos, connectionstyle="angle"
+    )
+
+    assert lbl[edge].x, lbl[edge].y == pytest.approx((label_pos, label_pos))
